;; 
;;  $Id: waveform.asm,v 1.12 2008/06/20 10:00:49 fvecoven Exp $
;; 
;;  Copyright (C) 2008 Frederic Vecoven
;; 
;;  This file is part of Pacsound
;; 
;;  Pacsound is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 3 of the License, or
;;  (at your option) any later version.
;; 
;;  Pacsound is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;; 
;;  You should have received a copy of the GNU General Public License
;;  along with this program. If not, see <http://www.gnu.org/licenses/>
;;


;
; PROM waveforms
;
; The waveforms are coming from chip 1M on pacman boards.
; The organization is 8 waveforms of 32 samples (4 bits each).
;
; The layout is re-organized for performance reasons :
;
; for (i = 0; i < 256; i++) {
; 	addr = ((i & 0x1f) << 3) | ((i & 0xe0) >> 5);
; 	new_prom[addr] = prom[i] << 4;
; }
;
;
; The table must be aligned on a 256-bytes boundary.
;

waveform_code	CODE	0x1000

prom_waveform_0
	GLOBAL	prom_waveform_0
        db      0x70, 0x70, 0x70, 0x70, 0x00, 0x70, 0x00, 0x00
        db      0x90, 0xc0, 0xa0, 0xd0, 0x80, 0x80, 0x10, 0x10
        db      0xa0, 0xe0, 0xc0, 0xb0, 0xf0, 0x60, 0x20, 0x20
        db      0xb0, 0xe0, 0xd0, 0x80, 0x70, 0x90, 0x30, 0x30
        db      0xc0, 0xd0, 0xe0, 0xb0, 0x10, 0x50, 0x40, 0x40
        db      0xd0, 0xb0, 0xd0, 0xd0, 0x80, 0xa0, 0x50, 0x50
        db      0xd0, 0x90, 0xc0, 0x90, 0xe0, 0x40, 0x60, 0x60
        db      0xe0, 0xa0, 0xa0, 0x60, 0x70, 0xb0, 0x70, 0x70
        db      0xe0, 0xb0, 0x70, 0xb0, 0x20, 0x30, 0x80, 0x80
        db      0xe0, 0xb0, 0x40, 0xe0, 0x80, 0xc0, 0x90, 0x90
        db      0xd0, 0xa0, 0x20, 0xc0, 0xd0, 0x20, 0xa0, 0xa0
        db      0xd0, 0x90, 0x10, 0x70, 0x70, 0xd0, 0xb0, 0xb0
        db      0xc0, 0x60, 0x00, 0x90, 0x30, 0x10, 0xc0, 0xc0
        db      0xb0, 0x40, 0x10, 0xa0, 0x80, 0xe0, 0xd0, 0xd0
        db      0xa0, 0x30, 0x20, 0x60, 0xc0, 0x00, 0xe0, 0xe0
        db      0x90, 0x50, 0x40, 0x20, 0x70, 0xf0, 0xf0, 0xf0
        db      0x70, 0x70, 0x70, 0x70, 0x40, 0x00, 0xf0, 0x00
        db      0x50, 0x90, 0xb0, 0xc0, 0x80, 0xf0, 0xe0, 0x10
        db      0x40, 0xb0, 0xd0, 0x80, 0xb0, 0x10, 0xd0, 0x20
        db      0x30, 0xa0, 0xe0, 0x40, 0x70, 0xe0, 0xc0, 0x30
        db      0x20, 0x80, 0xd0, 0x50, 0x50, 0x20, 0xb0, 0x40
        db      0x10, 0x50, 0xb0, 0x70, 0x80, 0xd0, 0xa0, 0x50
        db      0x10, 0x40, 0x70, 0x20, 0xa0, 0x30, 0x90, 0x60
        db      0x00, 0x30, 0x30, 0x00, 0x70, 0xc0, 0x80, 0x70
        db      0x00, 0x30, 0x10, 0x30, 0x60, 0x40, 0x70, 0x80
        db      0x00, 0x40, 0x00, 0x80, 0x80, 0xb0, 0x60, 0x90
        db      0x10, 0x50, 0x10, 0x50, 0x90, 0x50, 0x50, 0xa0
        db      0x10, 0x30, 0x30, 0x10, 0x70, 0xa0, 0x40, 0xb0
        db      0x20, 0x10, 0x70, 0x30, 0x70, 0x60, 0x30, 0xc0
        db      0x30, 0x00, 0xe0, 0x60, 0x80, 0x90, 0x20, 0xd0
        db      0x40, 0x00, 0x70, 0x30, 0x80, 0x70, 0x10, 0xe0
        db      0x50, 0x20, 0x00, 0x10, 0x70, 0x80, 0x00, 0xf0
        
effect_table_0
	GLOBAL	effect_table_0
	
; mspacman channel 1 effects (same for pacman)
	
	; 1  free life
        db	0x73, 0x20, 0x00, 0x0C, 0x00, 0x0A, 0x1f, 0x00
        ; 2  credit
	db	0x72, 0x20, 0xfb, 0x87, 0x00, 0x02, 0x0f, 0x00
  
; mspacman channels 2-3 effects
        
        ; 3  eat dot
        db	0x1c, 0x70, 0x8b, 0x08, 0x00, 0x01, 0x06, 0x00
        ; 4  eat fruit
        db	0x56, 0x0c, 0xff, 0x8c, 0x00, 0x02, 0x08, 0x00
        ; 5  eat ghost
        db	0x56, 0x00, 0x02, 0x0a, 0x07, 0x03, 0x0c, 0x00
        ; 6  mspacman death
        db	0x36, 0x38, 0xfe, 0x12, 0xf8, 0x04, 0x0f, 0xfc
        ; 7  bass (fruit moving)
        db	0x22, 0x01, 0x01, 0x06, 0x00, 0x01, 0x07, 0x00
     	; 8  siren 1 
        db	0x59, 0x01, 0x06, 0x08, 0x00, 0x00, 0x02, 0x00 
       	; 9  siren 2
        db	0x59, 0x01, 0x06, 0x09, 0x00, 0x00, 0x02, 0x00  
        ; 10 siren 3
        db	0x59, 0x02, 0x06, 0x0a, 0x00, 0x00, 0x02, 0x00
        ; 11 siren 4
        db	0x59, 0x03, 0x06, 0x0b, 0x00, 0x00, 0x02, 0x00
        ; 12 siren 5
        db	0x59, 0x04, 0x06, 0x0c, 0x00, 0x06, 0x02, 0x00
        ; 13 blue ghosts background
        db	0x24, 0x00, 0x06, 0x08, 0x02, 0x00, 0x0a, 0x00
        ; 14 background when a ghost is under eye form
        db	0x36, 0x07, 0x87, 0x6f, 0x00, 0x00, 0x04, 0x00
     
; pacman channel 2-3 effects
	
	; 15  eat dot 1
	db	0x42, 0x18, 0xfd, 0x06, 0x00, 0x01, 0x0c, 0x00
	; 16  eat dot 2
	db	0x42, 0x04, 0x03, 0x06, 0x00, 0x01, 0x0c, 0x00
	; 17  eat ghost
	db	0x56, 0x0c, 0xff, 0x8c, 0x00, 0x02, 0x0f, 0x00
	; 18
	db	0x05, 0x00, 0x02, 0x20, 0x00, 0x01, 0x0c, 0x00
	; 19  death
	db	0x41, 0x20, 0xff, 0x86, 0xfe, 0x1c, 0x0f, 0xff
	; 20
	db	0x70, 0x00, 0x01, 0x0c, 0x00, 0x01, 0x08, 0x00

        
 
             
        
        
wave_table_0
	GLOBAL	wave_table_0
	; 1
	db	high(pac_start_1), low(pac_start_1)
	; 2
	db	high(pac_start_2), low(pac_start_2)
	; 3
	db	high(mspac_start_1), low(mspac_start_1)
	; 4
	db	high(mspac_start_2), low(mspac_start_2)
	;5
	db	high(pac_act_1), low(pac_act_1)
	; 6
	db	high(pac_act_2), low(pac_act_2)
	; 7
	db	high(mspac_act1_1), low(mspac_act1_1)
	; 8
	db	high(mspac_act1_2), low(mspac_act1_2)
	; 9
	db	high(mspac_act2_1), low(mspac_act2_1)
	; 10
	db	high(mspac_act2_2), low(mspac_act2_2)
	; 11
	db	high(mspac_act3_1), low(mspac_act3_1)
	; 12
	db	high(mspac_act3_2), low(mspac_act3_2)	
	
	; pacman start (channel 1)
pac_start_1
	db	0xf1, 0x02, 0xf2, 0x03, 0xf3, 0x0f, 0xf4, 0x01
	db	0x82, 0x70, 0x69, 0x82, 0x70, 0x69, 0x83, 0x70
	db	0x6a, 0x83, 0x70, 0x6a, 0x82, 0x70, 0x69, 0x82
	db	0x70, 0x69, 0x89, 0x8b, 0x8d, 0x8e, 0xff
	
	; pacman start (channel 2)	
pac_start_2
	db	0xf1, 0x00, 0xf2, 0x02, 0xf3, 0x0f, 0xf4, 0x00
	db	0x42, 0x50, 0x4e, 0x50, 0x49, 0x50, 0x46, 0x50
	db	0x4e, 0x49, 0x70, 0x66, 0x70, 0x43, 0x50, 0x4f
	db	0x50, 0x4a, 0x50, 0x47, 0x50, 0x4f, 0x4a, 0x70
	db	0x67, 0x70, 0x42, 0x50, 0x4e, 0x50, 0x49, 0x50
	db	0x46, 0x50, 0x4e, 0x49, 0x70, 0x66, 0x70, 0x45
	db	0x46, 0x47, 0x50, 0x47, 0x48, 0x49, 0x50, 0x49
	db	0x4a, 0x4b, 0x50, 0x6e, 0xff
	
	; pacman act (channel 1)
pac_act_1
	db	0xf1, 0x02, 0xf2, 0x03, 0xf3, 0x0f, 0xf4, 0x01
pac_act_1_loop
	db	0x67, 0x50, 0x30, 0x47, 0x30, 0x67, 0x50, 0x30
	db	0x47, 0x30, 0x67, 0x50, 0x30, 0x47, 0x30, 0x4b
	db	0x10, 0x4c, 0x10, 0x4d, 0x10, 0x4e, 0x10, 0x67
	db	0x50, 0x30, 0x47, 0x30, 0x67, 0x50, 0x30, 0x47
	db	0x30, 0x67, 0x50, 0x30, 0x47, 0x30, 0x4b, 0x10
	db	0x4c, 0x10, 0x4d, 0x10, 0x4e, 0x10, 0x67, 0x50
	db	0x30, 0x47, 0x30, 0x67, 0x50, 0x30, 0x47, 0x30
	db	0x67, 0x50, 0x30, 0x47, 0x30, 0x4b, 0x10, 0x4c
	db	0x10, 0x4d, 0x10, 0x4e, 0x10, 0x77, 0x20, 0x4e
	db	0x10, 0x4d, 0x10, 0x4c, 0x10, 0x4a, 0x10, 0x47
	db	0x10, 0x46, 0x10, 0x65, 0x30, 0x66, 0x30, 0x67
	db	0x40, 0x70, 0xf0, high(pac_act_1_loop)
	db	low(pac_act_1_loop)
	
	; pacman act (channel 2)
pac_act_2
	db	0xf1, 0x01, 0xf2, 0x01, 0xf3, 0x0f, 0xf4, 0x00
pac_act_2_loop
	db	0x26, 0x67, 0x26, 0x67, 0x26, 0x67, 0x23, 0x44
	db	0x42, 0x47, 0x30, 0x67, 0x2a, 0x8b, 0x70, 0x26
	db	0x67, 0x26, 0x67, 0x26, 0x67, 0x23, 0x44, 0x42
	db	0x47, 0x30, 0x67, 0x23, 0x84, 0x70, 0x26, 0x67
	db	0x26, 0x67, 0x26, 0x67, 0x23, 0x44, 0x42, 0x47
	db	0x30, 0x67, 0x29, 0x6a, 0x2b, 0x6c, 0x30, 0x2c
	db	0x6d, 0x40, 0x2b, 0x6c, 0x29, 0x6a, 0x67, 0x20
	db	0x29, 0x6a, 0x40, 0x26, 0x87, 0x70
	db	0xf0, high(pac_act_2_loop), low(pac_act_2_loop)
	
	; mspacman start (channel 1)
mspac_start_1
	db	0xf1, 0x02, 0xf2, 0x03, 0xf3, 0x0a, 0xf4, 0x00
	db	0x50, 0x70, 0x86, 0x90, 0x81, 0x90, 0x86, 0x90
	db	0x68, 0x6a, 0x6b, 0x68, 0x6a, 0x68, 0x66, 0x6a
	db	0x68, 0x66, 0x65, 0x68, 0x86, 0x81, 0x86, 0xff
	
	; mspacman start (channel 2)
mspac_start_2
	db	0xf1, 0x00, 0xf2, 0x02, 0xf3, 0x0a, 0xf4, 0x00
	db	0x41, 0x43, 0x45, 0x86, 0x8a, 0x88, 0x8b, 0x6a
	db	0x6b, 0x71, 0x6a, 0x88, 0x8b, 0x6a, 0x6b, 0x71
	db	0x6a, 0x6b, 0x71, 0x73, 0x75, 0x96, 0x95, 0x96
	db	0xff
	
	; mspacman act 1 (channel 1)
mspac_act1_1
	db	0xf1, 0x03, 0xf2, 0x03, 0xf3, 0x0a, 0xf4, 0x02
	db	0x70, 0x66, 0x70, 0x46, 0x50, 0x86, 0x90, 0x70
	db	0x66, 0x70, 0x46, 0x50, 0x86, 0x90, 0x70, 0x66
	db	0x70, 0x46, 0x50, 0x86, 0x90, 0x70, 0x61, 0x70
	db	0x41, 0x50, 0x81, 0x90, 0xf4, 0x00, 0xa6, 0xa4
	db	0xa2, 0xa1, 0xf4, 0x01, 0x86, 0x89, 0x8b, 0x81
	db	0x74, 0x71, 0x6b, 0x69, 0xa6, 0xff
	
	; mspacman act 1 (channel 2)
mspac_act1_2
	db	0xf1, 0x00, 0xf2, 0x02, 0xf3, 0x0a, 0xf4, 0x00
	db	0x69, 0x6b, 0x69, 0x86, 0x61, 0x64, 0x65, 0x86
	db	0x86, 0x64, 0x66, 0x64, 0x61, 0x69, 0x6b, 0x69
	db	0x86, 0x61, 0x64, 0x64, 0xa1, 0x70, 0x71, 0x74
	db	0x75, 0x35, 0x76, 0x30, 0x50, 0x35, 0x76, 0x30
	db	0x50, 0x54, 0x56, 0x54, 0x51, 0x6b, 0x69, 0x6b
	db	0x69, 0x6b, 0x91, 0x6b, 0x69, 0x66, 0xf2, 0x01
	db	0x74, 0x76, 0x74, 0x71, 0x74, 0x71, 0x6b, 0x69
	db	0xa6, 0xa6, 0xff
	
   	; mspacman act 2 (channel 1)
mspac_act2_1
        db      0xf1, 0x03, 0xf2, 0x03, 0xf3, 0x0a, 0xf4, 0x02
        db      0x90, 0x7c, 0x7b, 0x7a, 0x79, 0x79, 0x78, 0x97
        db      0x76, 0x75, 0x74, 0x73, 0x73, 0x72, 0x91, 0xa8
        db      0x88, 0x60, 0x4a, 0x4c, 0x91, 0x95, 0x88, 0x95
        db      0x91, 0x95, 0x88, 0x95, 0x91, 0x95, 0x88, 0x95
        db      0x95, 0x98, 0x94, 0x97, 0x93, 0x96, 0x88, 0x96
        db      0x93, 0x96, 0x88, 0x96, 0x93, 0x96, 0x88, 0x96
        db      0xb6, 0xb3, 0x75, 0x76, 0x77, 0x78, 0x78, 0x75
        db      0x73, 0x68, 0x91, 0x95, 0x88, 0x95, 0x91, 0x95
        db      0x88, 0x95, 0x86, 0x96, 0x95, 0x92, 0x93, 0x8c
        db      0x8a, 0x88, 0x86, 0x90, 0x90, 0x96, 0x95, 0x90
        db      0x90, 0x86, 0x90, 0x96, 0x90, 0x96, 0x91, 0x88
        db      0x81, 0xff
	
	; mspacman act 2 (channel 2)
mspac_act2_2
        db      0xf1, 0x00, 0xf2, 0x02, 0xf3, 0x0a, 0xf4, 0x00
        db      0x88, 0x6c, 0x71, 0x72, 0x73, 0x73, 0x71, 0x93
        db      0x6c, 0x73, 0x75, 0x76, 0x76, 0x75, 0x96, 0x7c
        db      0x7a, 0x78, 0x76, 0x75, 0x96, 0x6c, 0x91, 0xa0
        db      0x88, 0x75, 0x76, 0x77, 0x78, 0x71, 0x73, 0x74
        db      0x75, 0x71, 0x75, 0x71, 0x68, 0x68, 0x65, 0x66
        db      0x67, 0xa8, 0xab, 0xac, 0x8c, 0x86, 0x76, 0x75
        db      0x6c, 0x71, 0x75, 0x73, 0x6b, 0x6c, 0x73, 0x76
        db      0x7a, 0x78, 0x78, 0x76, 0x73, 0x6c, 0xaa, 0xa8
        db      0x71, 0x73, 0x74, 0x75, 0x6a, 0x6b, 0x6c, 0x73
        db      0x75, 0x76, 0x77, 0x78, 0x71, 0x73, 0x74, 0x75
        db      0x71, 0x75, 0x71, 0x68, 0x48, 0x40, 0x68, 0x67
        db      0x68, 0xaa, 0xa9, 0xaa, 0x6a, 0x60, 0x8a, 0x76
        db      0x75, 0x73, 0x71, 0x71, 0x73, 0x95, 0x75, 0x73
        db      0x71, 0x68, 0x68, 0x61, 0x63, 0x6a, 0xa8, 0x6c
        db      0x76, 0x6a, 0x6c, 0x91, 0x90, 0x91, 0xff
        
	; mspacman act 3 (channel 1)
mspac_act3_1        
        db      0xf1, 0x02, 0xf2, 0x03, 0xf3, 0x0a, 0xf4, 0x02
        db      0x65, 0x90, 0x68, 0x70, 0x68, 0x67, 0x66, 0x65
        db      0x90, 0x61, 0x70, 0x61, 0x65, 0x68, 0x66, 0x90
        db      0x63, 0x90, 0x86, 0x90, 0x85, 0x90, 0x85, 0x70
        db      0x86, 0x68, 0x65, 0xff
        
 	; mspacman act 3 (channel 2)
mspac_act3_2          
        db      0xf1, 0x00, 0xf2, 0x02, 0xf3, 0x0a, 0xf4, 0x00
        db      0x65, 0x64, 0x65, 0x88, 0x67, 0x88, 0x61, 0x63
        db      0x64, 0x85, 0x64, 0x85, 0x6a, 0x69, 0x6a, 0x8c
        db      0x75, 0x93, 0x90, 0x91, 0x90, 0x91, 0x70, 0x8a
        db      0x68, 0x71, 0xff        
        
        
        END
        